/* Copyright 2020, Michele Santullo
 * This file is part of orotool.
 *
 * Orotool is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Orotool is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Orotool.  If not, see <http://www.gnu.org/licenses/>.
 */

#pragma once

#include <exception>
#if !defined(NDEBUG)
#	include <iostream>
#	include <typeinfo>
#endif

namespace eve {

class Context;
class Eventia;

class Event {
protected:
	explicit Event(const Context& ctx);
	virtual ~Event() noexcept;

	virtual void on_loop_stopping() noexcept = 0;
	void set_exception (std::exception_ptr ptr);

	template <typename EV> void stop_ifn(EV& eve);
	bool is_stopped() const noexcept;

private:
	void on_loop_stopping_local() noexcept;
#if !defined(NDEBUG)
	static std::string demangle(const char* name);
#endif

	unsigned int m_cb_id;
	Eventia* m_eventia;
	bool m_was_stopped;
};

template <typename EV>
inline void Event::stop_ifn (EV& eve) {
#if !defined(NDEBUG)
	const bool was_stopped = m_was_stopped;
#endif

	if (not m_was_stopped) {
		m_was_stopped = true;
		eve.stop();
	}

#if !defined(NDEBUG)
	std::cout << "Event stopped: " << demangle(typeid(EV).name()) <<
		" (was already stopped: " << std::boolalpha << was_stopped << ")\n";
#endif
}

} //namespace eve
