/* Copyright 2020, Michele Santullo
 * This file is part of orotool.
 *
 * Orotool is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Orotool is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Orotool.  If not, see <http://www.gnu.org/licenses/>.
 */

#pragma once

#include <memory>
#include <functional>
#include <exception>

namespace eve {

class Context;
class Event;

class Eventia {
	friend class Event;
public:
	Eventia();
	~Eventia() noexcept;

	std::function<void()> event_functor();

	template <typename T, typename... Args>
	T make_event(Args&&... args) {
		return T{this->context(), std::forward<Args>(args)...};
	}

	void stop() noexcept;
	void wait();
	void set_exception (std::exception_ptr ptr) noexcept;

private:
	struct LocalData;

	void lock_mutex_libev() noexcept;
	void unlock_mutex_libev() noexcept;
	const Context& context();
	void on_async();

	unsigned int register_stop_callback (std::function<void()>&& cb);
	void unregister_stop_callback (unsigned int id) noexcept;

	std::unique_ptr<LocalData> m_local;
};

} //namespace eve
