/* Copyright 2020, Michele Santullo
 * This file is part of orotool.
 *
 * Orotool is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Orotool is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Orotool.  If not, see <http://www.gnu.org/licenses/>.
 */

#pragma once

#include "event.hpp"
#include <memory>

namespace eve {

class Context;

class Timer : private Event {
public:
	Timer() = delete;
	Timer(Timer&&) = default;
	Timer(const Timer&) = delete;
	Timer (double delay, const Context& ctx);
	virtual ~Timer() noexcept;

	virtual void on_timer() = 0;

protected:
	void set_timer (double delay);
	using Event::set_exception;

private:
	struct LocalData;

	void on_timer_ev() noexcept;
	virtual void on_loop_stopping() noexcept override;

	std::unique_ptr<LocalData> m_local;
};

} //namespace eve
