/* Copyright 2020, Michele Santullo
 * This file is part of orotool.
 *
 * Orotool is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Orotool is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Orotool.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "eventia_thread_pool.hpp"
#include "eventia/eventia.hpp"
#include <cassert>

namespace duck {

EventiaThreadPool::EventiaThreadPool (eve::Eventia* eventia) :
	roar11::ThreadPool(),
	m_eventia(eventia)
{
	assert(m_eventia);
}

EventiaThreadPool::EventiaThreadPool (eve::Eventia* eventia, std::uint32_t num_threads) :
	roar11::ThreadPool(num_threads),
	m_eventia(eventia)
{
	assert(m_eventia);
}

EventiaThreadPool::~EventiaThreadPool() = default;

void EventiaThreadPool::wait() noexcept {
	this->destroy();
}

void EventiaThreadPool::set_exception (std::exception_ptr ptr) {
	m_eventia->set_exception(ptr);
}

} //namespace duck
