/* Copyright 2020, Michele Santullo
 * This file is part of orotool.
 *
 * Orotool is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Orotool is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Orotool.  If not, see <http://www.gnu.org/licenses/>.
 */

#pragma once

#include "http_response.hpp"
#include <simdjson.h>
#include <string_view>
#include <string>
#include <initializer_list>
#include <vector>
#include <utility>
#include <cstddef>

namespace nap {

class QuickRest {
public:
	typedef std::pair<std::string_view, std::string_view> HeaderPairView;

	explicit QuickRest(std::size_t body_padding=0);

	void add_headers (std::initializer_list<HeaderPairView> headers);
	void set_user_agent (std::string&& name);
	HttpResponse fetch (std::string_view url);

private:
	std::string m_user_agent;
	std::vector<std::string> m_header_lines;
	std::size_t m_body_padding;
};

} //namespace nap
