/* Copyright 2020, Michele Santullo
 * This file is part of orotool.
 *
 * Orotool is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Orotool is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Orotool.  If not, see <http://www.gnu.org/licenses/>.
 */

#pragma once

#include <stdexcept>
#include <array>
#include <string_view>

namespace nap {
struct HttpResponse;
} //namespace nap

namespace oro {

class ServerError : public std::runtime_error {
public:
	explicit ServerError (const nap::HttpResponse& resp);
	int error_code() const noexcept;

private:
	int m_err_code;
};

class ContentTypeError : public std::runtime_error {
public:
	ContentTypeError (std::string_view type, std::string_view subtype);
	std::string_view content_type() const noexcept;

private:
	std::array<char, 32> m_content_type;
};

class ConnectionError : public std::runtime_error {
public:
	ConnectionError (const std::string& reason, unsigned int code);

	unsigned int curl_code() const;

private:
	unsigned int m_curl_code;
};

} //namespace oro
