/* Copyright 2020, Michele Santullo
 * This file is part of orotool.
 *
 * Orotool is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Orotool is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Orotool.  If not, see <http://www.gnu.org/licenses/>.
 */

#pragma once

#include "datatypes.hpp"
#include "oro/dboperation.hpp"
#include "oro/source.hpp"
#include <memory>

namespace oro {

struct Items;
struct Icons;
struct Shops;
struct Creators;

class OriginsDB {
public:
	OriginsDB() = default;
	virtual ~OriginsDB() noexcept = default;

	virtual void update (const Items& items, const oro::Timestamp& next_update, const Source& source) = 0;
	virtual void update (const Icons& icons, const oro::Timestamp& next_update, const Source& source) = 0;
	virtual void update (const Shops& shops, const oro::Timestamp& next_update, const Source& source) = 0;
	virtual void update (const Creators& creat, const oro::Timestamp& next_update, const Source& source) = 0;

	virtual Timestamp next_access_time (DBOperation op) const = 0;
	virtual void update_access_time (const oro::Timestamp& next_update, DBOperation op) = 0;

	static std::unique_ptr<OriginsDB> make (std::string_view name, std::string_view param);
};
} //namespace oro
