/* Copyright 2020, Michele Santullo
 * This file is part of orotool.
 *
 * Orotool is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Orotool is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Orotool.  If not, see <http://www.gnu.org/licenses/>.
 */

#pragma once

#include "oro/originsdb.hpp"
#include <mutex>
#include <string_view>
#include <string>
#include <memory>

namespace SQLite {
class Database;
} //namespace SQLite

namespace oro {

class OriginsDBSQLite : public OriginsDB {
public:
	explicit OriginsDBSQLite(std::string_view path);
	virtual ~OriginsDBSQLite() noexcept;

	void update (const Items& items, const oro::Timestamp& next_update, const Source& source) override;
	void update (const Icons& icons, const oro::Timestamp& next_update, const Source& source) override;
	void update (const Shops& shops, const oro::Timestamp& next_update, const Source& source) override;
	void update (const Creators& creat, const oro::Timestamp& next_update, const Source& source) override;

	Timestamp next_access_time (DBOperation op) const override;
	void update_access_time (const oro::Timestamp& next_update, DBOperation op) override;

private:
	std::string m_path;
	std::mutex m_db_mutex;
	std::unique_ptr<SQLite::Database> m_db;
};

} //namespace oro
